﻿/*

File: SlopeChart.js

Abstract: JavaScript for Slope Chart widget

Version: 1.0

© Copyright 2007 Mulligan Software. All rights reserved.

*/

/*********************************/
// HIDING AND SHOWING PREFERENCES
/*********************************/


// Global variables for the info and done buttons; keeping these around lets us modify them later
var glassDoneButton;
var glassNoticeButton;
var glassOKButton;
var whiteInfoButton;
var backInfoButton;
var nonBreakingSpace;

// getLocalizedString() pulls a string out an array named localizedStrings.  Each language project
// directory in this widget contains a file named "localizedStrings.js", which, in turn, contains
// an array called localizedStrings.  This method queries the array of the file of whichever language
// has highest precidence, according to the International pane of System Preferences.

function getLocalizedString(key)
{
    try {
        var ret = localizedStrings[key];
        if (ret === undefined)
            ret = key;
        return ret;
    } catch (ex) {}
    return key;
}

// setup() is the onload handler for the widget. It calls constructors for the AppleInfoButton
// and AppleGlassButton classes that populate existing divs with their respective controls

function setup()
{
	glassDoneButton = new AppleGlassButton(document.getElementById("doneButton"), getLocalizedString("Done"), hidePrefs);
	glassOKButton = new AppleGlassButton(document.getElementById("okButton"), getLocalizedString("OK"), showPrefs);
	whiteInfoButton = new AppleInfoButton(document.getElementById("infoButton"), document.getElementById("front"), "white", "white", showPrefs);
	backInfoButton = new AppleInfoButton(document.getElementById("info2Button"), document.getElementById("back"), "white", "white", showNotice);
	nonBreakingSpace = document.getElementById('nonBreakingSpace').innerText;
	
	if(window.widget)		// always check to make sure that you are running in Dashboard
	{
		document.getElementById('chartTitle').innerText = getLocalizedString('Course Handicap Table');
		document.getElementById('hiHeader1').innerText = getLocalizedString('Handicap Index');
		document.getElementById('chHeader1').innerText = getLocalizedString('Course Handicap');
		document.getElementById('hiHeader2').innerText = getLocalizedString('Handicap Index');
		document.getElementById('chHeader2').innerText = getLocalizedString('Course Handicap');

		document.getElementById('courseLabel').innerText = getLocalizedString('Course:');
		document.getElementById('teeLabel').innerText = getLocalizedString('Tee:');
		document.getElementById('sexLabel').innerText = getLocalizedString('For:');
		document.getElementById('menRadio').innerText = getLocalizedString('Men');
		document.getElementById('womenRadio').innerText = getLocalizedString('Women');
		document.getElementById('slopeLabel').innerText = getLocalizedString('Slope:');
		document.getElementById('ratingLabel').innerText = getLocalizedString('Rating:');

		document.getElementById('noticeHead').innerText = getLocalizedString('Notice');
		document.getElementById('contentText').innerText = getLocalizedString('Content');

		var slope = 113;
		
		coursePref = widget.preferenceForKey(makeKey("course"));
		teePref = widget.preferenceForKey(makeKey("tee"));
		sexPref = widget.preferenceForKey(makeKey("sex"));
		ratingPref = widget.preferenceForKey(makeKey("rating"));
		slopePref = widget.preferenceForKey(makeKey("slope"));
		
		if (coursePref && coursePref.length > 0)  // if the retrieved preferences are not empty, restore them
		{
			document.getElementById("courseTitle").innerText = coursePref;
			document.getElementById("courseText").value = coursePref;
		}
		else
		{
			document.getElementById("courseTitle").innerText = getLocalizedString('Click info');
		}
		
		if (teePref && teePref.length > 0)  // if the retrieved preferences are not empty, restore them
		{
			document.getElementById("teeTitle").innerText = teePref;
			document.getElementById("teeText").value = teePref;
		}

		if (sexPref && sexPref == "women")
		{
			document.getElementById("wRadio").checked = true;
			document.getElementById("sexTitle").innerText = getLocalizedString("Women");
		}
		else
		{
			document.getElementById("mRadio").checked = true;
			document.getElementById("sexTitle").innerText = getLocalizedString("Men");
		}
		
		if (slopePref && slopePref.length > 0)  // if the retrieved preferences are not empty, restore them
		{
			slope = parseInt(slopePref);
		}
		updateSlopeTitle(parseInt(slope));
		document.getElementById("slopeText").value = slope;

		if (ratingPref && ratingPref.length > 0)  // if the retrieved preferences are not empty, restore them
		{
			updateRatingTitle(parseFloat(ratingPref));
			document.getElementById("ratingText").value = ratingPref;
		}

		rebuildChart();
	}
}

function sizeForLines(numLines)
{
	var newWidth = parseInt(document.defaultView.getComputedStyle(document.getElementById("top"), "").getPropertyValue("width"));
	var newHeight = 300;

	if (numLines && numLines > 0) 
	{
		//	Each line is 12px
		var midHeight = 12 * parseInt(numLines);
		newHeight = parseInt(document.defaultView.getComputedStyle(document.getElementById("top"), "").getPropertyValue("height")) + parseInt(document.defaultView.getComputedStyle(document.getElementById("bottom"), "").getPropertyValue("height")) + midHeight;
	}

	if(window.widget)
	{
		window.resizeTo(newWidth, newHeight);		// resize the widget window
		document.getElementById("front").style.height = newHeight;
	}
}

function chartArray(slope, forWomen)
{
	var results = new Array;
	var maxIndex = 36.4;
	if (forWomen)
		maxIndex = 40.4;
	var maxHandicap = parseInt((slope * maxIndex / 113.0) + 0.5) + 1;
	
	for (var i = -4; i < maxHandicap; i++) {

		var floatHdcp = parseFloat(i < 0 ? -i : i);
		var low = parseInt(((floatHdcp - 0.5) * 1130.0 / slope) + 0.999) / 10.0;
		var high = parseInt(((floatHdcp + 0.5) * 1130.0 / slope) - 0.001) / 10.0;

		if (high > maxIndex)
			high = maxIndex;
			
		if (parseInt(high) == high)
			high = high + ".0";
		if (parseInt(low) == low)
			low = low + ".0";
			
		if (i < 0) {
			var tmp = low;
			low = "+" + high;
			high = "+" + tmp;
		} else if (i == 0) {
			low = "+" + high;
		}
		results[results.length] = low + "/" + high + "/" + (i < 0 ? ("+" + (-i)) : i);
	}
	
	return results;
}

function rebuildChart()
{
	var slope = 113;
	var forWomen = false;
	if (window.widget) {
		var slopePref = widget.preferenceForKey(makeKey("slope"));
		if (slopePref && (slopePref.length > 0))
			slope = parseInt(slopePref);
		var sexPref = widget.preferenceForKey(makeKey("sex"));
		if (sexPref && (sexPref == "women"))
			forWomen = true;
	}

	var rows = updateChart(slope, forWomen);
	sizeForLines(rows);
}

function updateChart(slope, forWomen)
{
	var toText = getLocalizedString('to');
	var tableRows = document.getElementById("slopeTable").getElementsByTagName("tr");
	var chartData = chartArray(slope, forWomen);
	var chartItems = chartData.length;
	var chartRows = parseInt((chartItems + 1) / 2.0);

	for (var row = 0; row < chartRows; row++) {
		var theTR = tableRows[row];
		var theTDs = theTR.getElementsByTagName("td");
		var theData = chartData[row];
		var parts = theData.split("/");
		theTDs[0].innerText = parts[0] + nonBreakingSpace + nonBreakingSpace + toText + nonBreakingSpace + nonBreakingSpace + parts[1];
		theTDs[1].innerText = parts[2];

		if ((row + chartRows) < chartItems) {
			theData = chartData[row + chartRows];
			parts = theData.split("/");
			if (parts[0] == parts[1])
				theTDs[2].innerText = parts[0];
			else
				theTDs[2].innerText = parts[0] + nonBreakingSpace + nonBreakingSpace + toText + nonBreakingSpace + nonBreakingSpace + parts[1];
			theTDs[3].innerText = parts[2];
		} else {
			theTDs[2].innerText = nonBreakingSpace;
			theTDs[3].innerText = nonBreakingSpace;
		}
	}

	return chartRows;
}

// changeCourse() is called whenever the course text field is changed in the widget.  It queries the
// text field, applies the change to the widget, and saves the preference.

function changeCourse(elem)
{	
	var string = elem.value;
	
	document.getElementById("courseTitle").innerText = string;		// change the course title to entered string
	if(window.widget)
	{
		if (string.length > 0)
			widget.setPreferenceForKey(string, makeKey("course"));	// and save the new preference to disk
		else
			widget.setPreferenceForKey(null, makeKey("course"));	// and save the new preference to disk
	}
} 

// changeTee() is called whenever the tee text field is changed in the widget.  It queries the
// text field, applies the change to the widget, and saves the preference.

function changeTee(elem)
{	
	var string = elem.value;
	
	document.getElementById("teeTitle").innerText = string;		// change the tee text to entered string
	if(window.widget)
	{
		if (string.length > 0)
			widget.setPreferenceForKey(string, makeKey("tee"));	// and save the new preference to disk
		else
			widget.setPreferenceForKey(null, makeKey("tee"));	// and save the new preference to disk
	}
} 

function changeSexTo(index)
{	
	var title = document.getElementById("sexTitle");
	
	switch(index)
	{
		case 0: 			// ("Men")	
			title.innerText = getLocalizedString("Men");		// change the front text
			if(window.widget)
				widget.setPreferenceForKey("men", makeKey("sex"));	// and save the new preference to disk
			break;
		case 1:				// ("Women")
			title.innerText = getLocalizedString("Women");		// change the front text
			if(window.widget)
				widget.setPreferenceForKey("women", makeKey("sex"));	// and save the new preference to disk
			break;
	}
} 

// changeSlope() is called whenever the slope text field is changed in the widget.  It queries the
// text field, applies the change to the widget, and saves the preference.

function changeSlope(elem)
{	
	var slope = parseInt(elem.value);
	updateSlopeTitle(slope);
	if (window.widget) {
		widget.setPreferenceForKey((((isNaN(slope)) || (slope < 55) || (slope > 155)) ? null : slope + ""), makeKey("slope"));	// Remove/save the preference
	}
} 

function updateSlopeTitle(slope)
{
	var realSlope = (((parseInt(slope) > 54) && (parseInt(slope) < 156)) ? slope : 113);
	document.getElementById("slopeTitle").innerText = getLocalizedString("Slope Rating") + " - " + realSlope;
}

function changeRating(elem)
{	
	var rating = parseFloat(elem.value);
	if (isNaN(rating) || (rating < 45.0) || (rating > 85.0)) {
		if(window.widget)
			widget.setPreferenceForKey(null, makeKey("rating"));	// and remove preference from disk
	} else {
		rating = parseFloat(parseInt((rating * 10.0) + 0.5)) / 10.0;	//	Strip extra digits
		if(window.widget)
			widget.setPreferenceForKey(rating + "", makeKey("rating"));	// and save the new preference to disk
	}
	updateRatingTitle(rating);
} 

function updateRatingTitle(rating)
{
	if (isNaN(rating) || rating < 45.0 || rating > 85.0) {
		document.getElementById("ratingTitle").innerText = "";
	} else {
		rating = parseFloat(parseInt((rating * 10.0) + 0.5)) / 10.0;
		if (parseInt(rating) == rating)
			rating = rating + ".0";
		document.getElementById("ratingTitle").innerText = getLocalizedString("Course Rating") + " - " + rating;
	}
}

// showPrefs() is called when the preferences flipper is clicked upon.  It freezes the front of the widget,
// hides the front div, unhides the back div, and then flips the widget over.

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	var notice = document.getElementById("notice");
	
	if (window.widget)
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	
	front.style.display="none";		// hide the front
	notice.style.display="none";	// hide the notice
	back.style.display="block";		// show the back
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget over	

}


// hidePrefs() is called by the done button on the back side of the widget.  It performs the opposite transition
// as showPrefs() does.

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	var notice = document.getElementById("notice");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	
	back.style.display="none";			// hide the back
	notice.style.display="none";		// hide the notice
	front.style.display="block";		// show the front
	
	rebuildChart();
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
}

// showNotice() is called by ???.  It hides the notice back and shows the notice

function showNotice()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	var notice = document.getElementById("notice");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	
	back.style.display="none";		//	hide the back
	front.style.display="none";		//	and the front
	notice.style.display="block";		//	show the notice
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
}

// makeKey makes the widget multi-instance aware

function makeKey(key)
{
	return (widget.identifier + "-" + key);
}


/***************/
// WIDGET EVENTS
/***************/

// removed is called when the widget is removed from the Dashboard

function removed()
{
	widget.setPreferenceForKey(null,makeKey("course"));
	widget.setPreferenceForKey(null,makeKey("tee"));
	widget.setPreferenceForKey(null,makeKey("sex"));
	widget.setPreferenceForKey(null,makeKey("rating"));
	widget.setPreferenceForKey(null,makeKey("slope"));
}

if(window.widget)
{
	widget.onremove = removed;
}	
